using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace ld30
{
    public class Game : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public static int ScreenWidth = 800;
        public static int ScreenHeight = 800;

        World up, down;

        public Game()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            IsFixedTimeStep = false; // allow 144 hz + NOT frying CPU!
        }

        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = ScreenWidth;
            graphics.PreferredBackBufferHeight = ScreenHeight;
            graphics.ApplyChanges();

            base.Initialize();
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Texture2D comet = Content.Load<Texture2D>("textures\\meteroid");
            up = new World(0, 400, Content.Load<Texture2D>("textures\\plane1"), comet, 
                Sprite.Generate(Color.Red, 10, 4, GraphicsDevice));
            down = new World(400, 800, Content.Load<Texture2D>("textures\\plane2"), comet,
                Sprite.Generate(Color.Blue, 10, 4, GraphicsDevice));
        }


        protected override void Update(GameTime gameTime)
        {
            KeyboardState state = Keyboard.GetState();

            if (state.IsKeyDown(Keys.Escape)) this.Exit();

            up.Update(gameTime, state.IsKeyDown(Keys.W), state.IsKeyDown(Keys.S), state.IsKeyDown(Keys.A), state.IsKeyDown(Keys.D), state.IsKeyDown(Keys.Space));
            down.Update(gameTime, state.IsKeyDown(Keys.Up), state.IsKeyDown(Keys.Down), state.IsKeyDown(Keys.Left), state.IsKeyDown(Keys.Right), state.IsKeyDown(Keys.NumPad0));

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);

            spriteBatch.Begin();
            up.Draw(spriteBatch);
            down.Draw(spriteBatch);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
