﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Collections.Generic;


namespace ld30
{
    class World
    {
        Random ran;

        int yLocation = 0;
        int maxYLocation = 0;

        int wave = 0;
        int totalComets; // comets who need to spawn this wave

        List<int> cometsToSpawn; // lists are not my favorite thing, but, in this game it's pretty handy.
        List<int> currentComets; // ugh. I am starting to dislike them now! I will do this differently some other time!
        List<Bullet> bullets = new List<Bullet>();

        float bulletTime = 0f;
        int bulletDamage = 30;
        float bulletRespawnTime = 100;

        int maxComets; // max comets in screen/game
        float deltaTime = 0;
        float cometRespawnCount = 200; // ms
        float cometSpeed = 1.8f; // pix / sec

        Vector2 cometOrgin;
        Vector2[] comets;
        float[] cometRotations;
        int[] cometHealth; // future
        Color[] cometColors; // future, when hit, ya know.

        Vector2 planePos;
        int planeStep = 4;

        Texture2D planeTex;
        Texture2D cometTex;
        Texture2D bulletTex;

        // importing twice the same textures is not very efficient, but doing it anyways
        public World(int y, int maxY, Texture2D plane, Texture2D comet, Texture2D bullet)
        {
            this.yLocation = y;
            this.maxYLocation = maxY;
            this.planeTex = plane;
            this.cometTex = comet;
            this.bulletTex = bullet;

            planePos = new Vector2(10, (y + maxY) / 2 + planeTex.Height / 2);

            ran = new Random(Guid.NewGuid().GetHashCode() + y - maxY);
            cometOrgin = new Vector2(comet.Width / 2, comet.Height / 2);
            NextWave();
        }

        public void NextWave()
        {
            wave++;

            int twave = wave * wave;
            totalComets = twave * 3 + 10;
            maxComets = twave / 2 * 8 + 6;
            cometRespawnCount = (maxComets / (twave * 2)) * 60;
            comets = new Vector2[maxComets];
            cometRotations = new float[maxComets];
            cometHealth = new int[maxComets];
            currentComets = new List<int>(maxComets);
            cometsToSpawn = new List<int>(maxComets);
            
            for (int i = 0; i < maxComets; i++)
            {
                comets[i].X = -1000;
                cometsToSpawn.Add(i);// CreateComet(i);
            }
        }

        // stupid bugs!
        public void Update(GameTime gt, bool up, bool down, bool left, bool right, bool fire)
        {
            deltaTime += (float)gt.ElapsedGameTime.TotalMilliseconds;
            bulletTime += (float)gt.ElapsedGameTime.TotalMilliseconds; 

            if (up && planePos.Y - planeStep > yLocation) planePos.Y -= planeStep;
            else if (down && planePos.Y + planeTex.Height + planeStep < maxYLocation) planePos.Y += planeStep;

            if (left && planePos.X - planeStep > 0) planePos.X -= planeStep;
            else if (right && planePos.X + planeTex.Width + planeStep < Game.ScreenWidth) planePos.X += planeStep;

            if (fire && bulletTime >= bulletRespawnTime)
            {
                bulletTime = 0f;
                bullets.Add(new Bullet(new Vector2(planePos.X + planeTex.Width / 2, planePos.Y + planeTex.Height / 2),
                    new Vector2(Game.ScreenWidth, planePos.Y + planeTex.Height / 2 + ran.Next(-21, 21))));
            }

            List<int> removeList = new List<int>();
            List<int> cometsInScreen = new List<int>();
            for (int i = 0; i < currentComets.Count; i++)
            {
                comets[currentComets[i]].X -= cometSpeed * 60 * (float)gt.ElapsedGameTime.TotalSeconds;
                cometRotations[currentComets[i]] += 0.001f;

                // remove when out of screen (+ 5 is for that it doesn't despawn to soon)
                if (comets[currentComets[i]].X + cometTex.Width < 0)
                {
                    if (totalComets > maxComets) cometsToSpawn.Add(currentComets[i]);
                    removeList.Add(currentComets[i]);
                    totalComets--;

                    if (totalComets == 0)
                    {
                        NextWave();
                        removeList = new List<int>(); // you have no chance to ruin my game!
                        break;
                    }
                }
                else if (comets[currentComets[i]].X < Game.ScreenWidth)
                {
                    if (IntersectsComet(currentComets[i], planePos, planeTex.Width, planeTex.Height))
                    {
                        planePos = Vector2.Zero;

                        // copying code here
                        if (totalComets > maxComets) cometsToSpawn.Add(currentComets[i]);
                        removeList.Add(currentComets[i]);
                        totalComets--;

                        // make sure he vanishes
                        comets[currentComets[i]].X = -1000;

                        if (totalComets == 0)
                        {
                            NextWave();
                            removeList = new List<int>(); // you have no chance to ruin my game!
                            break;
                        }
                    }
                    else cometsInScreen.Add(currentComets[i]);
                }
            }

            // remove properly.
            for (int i = 0; i < removeList.Count; i++)
            {
                currentComets.Remove(removeList[i]);
            }

            removeList = new List<int>();
            List<int> bulletRemoveList = new List<int>();
            for (int i = 0; i < bullets.Count; i++)
            {
                bullets[i].Update((float)gt.ElapsedGameTime.TotalSeconds, 1.8f);
                if (bullets[i].pos.X + bulletTex.Width > Game.ScreenWidth)
                {
                    bulletRemoveList.Add(i);
                }
                else
                {
                    for (int a = 0; a < cometsInScreen.Count; a++)
                    {
                        if (IntersectsComet(cometsInScreen[a], bullets[i].pos, bulletTex.Width, bulletTex.Height))
                        {
                            bulletRemoveList.Add(i);
                            cometHealth[cometsInScreen[a]] -= bulletDamage;

                            if (cometHealth[cometsInScreen[a]] <= 0f)
                            {
                                removeList.Add(cometsInScreen[a]);

                                // copyed code again
                                if (totalComets > maxComets) cometsToSpawn.Add(cometsInScreen[a]);
                                removeList.Add(cometsInScreen[a]);
                                totalComets--;

                                // make sure he vanishes
                                
                                if (totalComets == 0)
                                {
                                    NextWave();
                                    bulletRemoveList = new List<int>();
                                    removeList = new List<int>(); // you have no chance to ruin my game!
                                    break;
                                }

                                comets[cometsInScreen[a]].X = -1000;
                                cometsInScreen.RemoveAt(a);
                            }
                            break;
                        }
                    }
                }
            }

            // remove bullets
            for (int i = bulletRemoveList.Count - 1; i >= 0; i--)
            {
                 bullets.RemoveAt(bulletRemoveList[i]);
            }

            // remove properly.
            for (int i = 0; i < removeList.Count; i++)
            {
                currentComets.Remove(removeList[i]);
            }

            // Wow... So much 0 here!
            if (deltaTime >= cometRespawnCount && cometsToSpawn.Count > 0)
            {
                deltaTime = 0;

                CreateComet(cometsToSpawn[0]);
                cometsToSpawn.RemoveAt(0);
            }
        }

        // Woo! Hopefully this works!
        bool IntersectsComet(int n, Vector2 pos, int width, int height)
        {
            return (pos.X + width > comets[n].X - cometOrgin.X && pos.X < comets[n].X + cometOrgin.X &&
                pos.Y + height > comets[n].Y - cometOrgin.Y && pos.Y < comets[n].Y + cometOrgin.Y);
        }

        void CreateComet(int n)
        {
            comets[n] = new Vector2(Game.ScreenWidth + ran.Next(0, Game.ScreenWidth + (int)cometOrgin.X), yLocation + ran.Next(0, maxYLocation - (int)cometOrgin.Y));
            cometRotations[n] = ((float)ran.Next(0, 1000) / 1000);
            cometHealth[n] = 100;
            currentComets.Add(n);
        }

        public void Draw(SpriteBatch sb)
        {
            for (int i = 0; i < comets.Length; i++)
                //if (comets[i] != null)
                    sb.Draw(cometTex, comets[i], null, Color.WhiteSmoke, cometRotations[i], Vector2.Zero, 1f, SpriteEffects.None, 0);

            for (int i = 0; i < bullets.Count; i++)
            {
                bullets[i].Draw(sb, bulletTex);
            }

            sb.Draw(planeTex, planePos, Color.White);
        }
    }
}
